package com.fr.design.chart.series.SeriesCondition.impl;

import com.fr.base.Utils;
import com.fr.chart.web.ChartHyperRelateCellLink;
import com.fr.design.gui.columnrow.ColumnRowVerticalPane;
import com.fr.design.gui.frpane.ReportletParameterViewPane;
import com.fr.design.gui.itextfield.UITextField;
import com.fr.design.hyperlink.AbstractHyperLinkPane;
import com.fr.design.layout.FRGUIPaneFactory;
import com.fr.design.utils.gui.GUICoreUtils;
import com.fr.stable.ColumnRow;
import com.fr.stable.ParameterProvider;

import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.HashMap;
import java.util.List;

/**
 * @author kunsnat E-mail:kunsnat@gmail.com
 * @version 创建时间：2011-12-28 下午02:51:10
 *          类说明: 图表超链 -- 关联单元格图表.
 */
public class ChartHyperRelateCellLinkPane extends AbstractHyperLinkPane<ChartHyperRelateCellLink> {
    private static final long serialVersionUID = 7874948047886548990L;

    private UITextField itemNameTextField;
    private ColumnRowVerticalPane colRowPane;
    private ReportletParameterViewPane parameterViewPane;

    public ChartHyperRelateCellLinkPane() {
        this.initComponent();
    }

    public ChartHyperRelateCellLinkPane(HashMap hyperLinkEditorMap, boolean needRenamePane) {
        super(hyperLinkEditorMap, needRenamePane);
        this.initComponent();
    }

    private void initComponent() {
        this.setLayout(FRGUIPaneFactory.createM_BorderLayout());
        if (needRenamePane()) {
            itemNameTextField = new UITextField();
            this.add(GUICoreUtils.createNamedPane(itemNameTextField, com.fr.design.i18n.Toolkit.i18nText("Fine-Design_Basic_Name") + ":"), BorderLayout.NORTH);
        }

        JPanel centerPane = FRGUIPaneFactory.createBorderLayout_L_Pane();
        this.add(centerPane, BorderLayout.CENTER);

        centerPane.setBorder(GUICoreUtils.createTitledBorder(com.fr.design.i18n.Toolkit.i18nText("Fine-Design_Chart_Hyper_Related_Cell")));

        colRowPane = new ColumnRowVerticalPane();
        centerPane.add(colRowPane, BorderLayout.NORTH);

        addPaneInCenter(centerPane);

        parameterViewPane = new ReportletParameterViewPane(getChartParaType(), getValueEditorPane(), getValueEditorPane());
        parameterViewPane.setBorder(GUICoreUtils.createTitledBorder(com.fr.design.i18n.Toolkit.i18nText("Fine-Design_Basic_Parameter")));
        parameterViewPane.setPreferredSize(new Dimension(500, 200));
        this.add(parameterViewPane, BorderLayout.SOUTH);
    }

    protected void addPaneInCenter(JPanel centerPane) {

    }

    @Override
    public void populateBean(ChartHyperRelateCellLink ob) {
        if (ob == null) {
            return;
        }
        if (itemNameTextField != null) {
            itemNameTextField.setText(ob.getItemName());
        }
        if (ob.getRelateCCName() != null) {
            ColumnRow colRow = ColumnRow.valueOf(ob.getRelateCCName());
            colRowPane.populate(colRow);
        } else {
            colRowPane.populate(ColumnRow.valueOf("A1"));
        }

        List parameterList = this.parameterViewPane.update();
        parameterList.clear();

        ParameterProvider[] parameters = ob.getParameters();
        parameterViewPane.populate(parameters);
    }

    @Override
    public ChartHyperRelateCellLink updateBean() {
        ChartHyperRelateCellLink chartLink = new ChartHyperRelateCellLink();
        updateBean(chartLink);
        if (itemNameTextField != null) {
            chartLink.setItemName(this.itemNameTextField.getText());
        }
        return chartLink;
    }

    /**
     * 属性表 对应update
     */
    public void updateBean(ChartHyperRelateCellLink chartLink) {
        chartLink.setRelateCCName(Utils.objectToString(colRowPane.update()));

        List parameterList = this.parameterViewPane.update();
        if (parameterList != null && !parameterList.isEmpty()) {
            ParameterProvider[] parameters = new ParameterProvider[parameterList.size()];
            parameterList.toArray(parameters);

            chartLink.setParameters(parameters);
        } else {
            chartLink.setParameters(null);
        }
        if (itemNameTextField != null) {
            chartLink.setItemName(this.itemNameTextField.getText());
        }
    }

    @Override
    public String title4PopupWindow() {
        return com.fr.design.i18n.Toolkit.i18nText("Fine-Design_Chart_Hyper_Related_Cell");
    }

    public static class ChartNoRename extends ChartHyperRelateCellLinkPane {
        public ChartNoRename() {
            super();
        }

        public ChartNoRename(HashMap hyperLinkEditorMap, boolean needRenamePane) {
            super(hyperLinkEditorMap, needRenamePane);
        }

        protected boolean needRenamePane() {
            return false;
        }
    }

    public static class ChartHasAnimateType extends ChartNoRename {

        public ChartHasAnimateType() {
            super();
        }

        public ChartHasAnimateType(HashMap hyperLinkEditorMap, boolean needRenamePane) {
            super(hyperLinkEditorMap, needRenamePane);
        }

        @Override
        protected void addPaneInCenter(JPanel centerPane) {
            centerPane.add(createAnimateTypeUIButtonGroup(), BorderLayout.CENTER);
        }

        @Override
        public void populateBean(ChartHyperRelateCellLink ob) {
            super.populateBean(ob);
            populateAnimateType(ob.getAnimateType());
        }

        @Override
        public void updateBean(ChartHyperRelateCellLink chartLink) {
            super.updateBean(chartLink);
            chartLink.setAnimateType(updateAnimateType());
        }
    }

}
